@@ -1,10 +1,20 @@
 /* 
- * ### Copyright (C) 2008 Michael Fuchs ###
- * ### All Rights Reserved.             ###
+ * Copyright (C) 2001-2013 Michael Fuchs
  *
- * Author: Michael Fuchs
- * E-Mail: michael.fuchs@unico-group.com
- * URL:    http://www.michael-a-fuchs.de
+ * This file is part of herold.
+ * 
+ * herold is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * herold is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with herold.  If not, see <http://www.gnu.org/licenses/>.  
  */
 package org.dbdoclet.herold;
 
@@ -34,12 +44,19 @@ import org.dbdoclet.option.StringOption;
 import org.dbdoclet.service.FileServices;
 import org.dbdoclet.service.ResourceServices;
 import org.dbdoclet.service.StringServices;
+import org.dbdoclet.trafo.AbstractTrafoService;
+import org.dbdoclet.trafo.TrafoConstants;
 import org.dbdoclet.trafo.TrafoException;
 import org.dbdoclet.trafo.TrafoExceptionHandler;
+import org.dbdoclet.trafo.TrafoResult;
 import org.dbdoclet.trafo.TrafoScriptManager;
-import org.dbdoclet.trafo.html.docbook.DbtConstants;
-import org.dbdoclet.trafo.html.docbook.DocBookTransformer;
+import org.dbdoclet.trafo.html.dita.HtmlDitaTrafo;
+import org.dbdoclet.trafo.html.docbook.HtmlDocBookTrafo;
+import org.dbdoclet.trafo.param.BooleanParam;
+import org.dbdoclet.trafo.param.TextParam;
+import org.dbdoclet.trafo.script.Namespace;
 import org.dbdoclet.trafo.script.Script;
+import org.dbdoclet.trafo.script.Section;
 
 /**
  * The class <code>Herold</code> implements a console converter to transform
@@ -50,19 +67,208 @@ import org.dbdoclet.trafo.script.Script;
  */
 public class Herold {
 
-	private static Log logger = LogFactory.getLog(Herold.class);
+	public enum OutputFormat {
+		DITA, DocBook
+	}
 
-	private static FileOption optIn;
-	private static FileOption optOut;
-	private static ResourceBundle res = ResourceBundle
+	private static Log logger = LogFactory.getLog(Herold.class);;
+
+	private int exitCode;
+	private FileOption optIn;
+	private FileOption optOut;
+	private OutputFormat outputFormat;
+	private ResourceBundle res = ResourceBundle
 			.getBundle("org/dbdoclet/herold/Resources");
+	private boolean systemExitEnabled = true;
 	private boolean verbose = false;
-	private static boolean systemExitEnabled = true;
 
-	private static int exitCode;
+	private static String getVersion() {
+
+		Package p = Herold.class.getPackage();
+		return p.getImplementationVersion();
+	}
 
 	public static void main(String[] args) {
 
+		Herold herold = new Herold();
+		herold.execute(args);
+	}
+
+	private static void printLicense() throws IOException {
+
+		URL url = ResourceServices
+				.getResourceAsUrl("/org/dbdoclet/herold/COPYING");
+
+		if (url != null) {
+			BufferedReader reader = new BufferedReader(new InputStreamReader(
+					url.openStream()));
+			String line = reader.readLine();
+
+			while (line != null) {
+				println(line);
+				line = reader.readLine();
+			}
+
+			reader.close();
+		} else {
+			logger.fatal("Can't find resource for license!");
+		}
+	}
+
+	private static void println(String str) {
+		System.out.println(str);
+	}
+
+	private static void printVersion() {
+
+		println("herold version \"" + getVersion() + "\"");
+	}
+
+	public void convert(File htmlFile, File xmlFile) throws TrafoException,
+			FileNotFoundException {
+
+		Script script = new Script();
+		convert(new FileInputStream(htmlFile), new FileOutputStream(xmlFile), xmlFile,
+				script);
+	}
+
+	public void convert(InputStream in, OutputStream out, File systemId, Script script)
+			throws TrafoException {
+
+		AbstractTrafoService trafo = new HtmlDocBookTrafo();
+
+		if (outputFormat == OutputFormat.DITA) {
+			trafo = new HtmlDitaTrafo();
+		}
+
+		trafo.setSystemId(systemId);
+		trafo.setInputStream(in);
+		trafo.setOutputStream(out);
+
+		TrafoResult result = null;
+		if (verbose == true) {
+			trafo.addProgressListener(new ConsoleProgressListener(false));
+			result = trafo.transform(script);
+		} else {
+			result = trafo.transform(script);
+		}
+
+		if (result.isFailed()) {
+			System.err.print(result.toString());
+		}
+
+		if (verbose == true) {
+			System.out.println();
+		}
+	}
+
+	private OptionList createOptionList(String[] args) {
+
+		Option<?> option;
+		SelectOption selopt;
+		BooleanOption bopt;
+		StringOption sopt;
+
+		OptionList options = new OptionList(args);
+
+		// docbook-add-index
+		bopt = new BooleanOption(TrafoConstants.SECTION_DOCBOOK.toLowerCase()
+				+ "-" + TrafoConstants.PARAM_ADD_INDEX, "x");
+		bopt.setValue(false);
+		options.add(bopt);
+
+		// docbook-encoding
+		sopt = new StringOption(TrafoConstants.SECTION_DOCBOOK.toLowerCase()
+				+ "-" + TrafoConstants.PARAM_ENCODING, "d");
+		sopt.setDefault("UTF-8");
+		options.add(sopt);
+
+		// decompose-tables
+		bopt = new BooleanOption(TrafoConstants.SECTION_DOCBOOK.toLowerCase()
+				+ "-" + TrafoConstants.PARAM_DECOMPOSE_TABLES, "T");
+		bopt.setDefault(false);
+		options.add(bopt);
+
+		// document-element
+		selopt = new SelectOption(TrafoConstants.SECTION_DOCBOOK.toLowerCase()
+				+ "-" + TrafoConstants.PARAM_DOCUMENT_ELEMENT, "r");
+
+		String[] optv2 = { "article", "book", "reference", "part", "chapter",
+				"section" };
+		selopt.setList(optv2);
+		selopt.setDefault("article");
+		options.add(selopt);
+
+		// title
+		sopt = new StringOption(TrafoConstants.SECTION_DOCBOOK.toLowerCase()
+				+ "-" + TrafoConstants.PARAM_TITLE, "t");
+		sopt.setDefault("http://www.dbdoclet.org/herold");
+		options.add(sopt);
+
+		// source-encoding
+		sopt = new StringOption(TrafoConstants.SECTION_HTML.toLowerCase() + "-"
+				+ TrafoConstants.PARAM_ENCODING, "s");
+		sopt.setDefault("UTF-8");
+		options.add(sopt);
+
+		// in
+		optIn = new FileOption("in", "i");
+		optIn.isExisting(true);
+		options.add(optIn);
+
+		// loglevel
+		SelectOption optLogLevel = new SelectOption("logging-level", "l");
+		optLogLevel.setList(new String[] { "debug5", "debug4", "debug3",
+				"debug2", "debug", "info", "warn", "error", "fatal" });
+		optLogLevel.setDefault("error");
+		options.add(optLogLevel);
+
+		// out
+		optOut = new FileOption("out", "o");
+		options.add(optOut);
+
+		StringOption optProfile = new StringOption("profile", "p");
+		options.add(optProfile);
+
+		// verbose
+		option = new BooleanOption("verbose", "v");
+		options.add(option);
+
+		return options;
+	}
+
+	public OutputFormat getOutputFormat() {
+		return outputFormat;
+	}
+
+	public void setOutputFormat(OutputFormat outputFormat) {
+		this.outputFormat = outputFormat;
+	}
+
+	private OutputStream createOutputStream() throws FileNotFoundException {
+
+		OutputStream out;
+
+		if (optOut.isUnset()) {
+
+			out = System.out;
+
+		} else {
+
+			File outDir = optOut.getValue().getParentFile();
+
+			if (outDir != null && outDir.exists() == false) {
+				outDir.mkdir();
+			}
+
+			out = new FileOutputStream(optOut.getValue());
+		}
+
+		return out;
+	}
+
+	public void execute(String[] args) {
+
 		OptionList options = null;
 		exitCode = 0;
 
@@ -107,8 +313,6 @@ public class Herold {
 				throw new OptionException(options.getError());
 			}
 
-			Herold converter = new Herold();
-
 			StringOption optProfile = (StringOption) options
 					.getOption("profile");
 			File profileFile = findProfile(optProfile);
@@ -118,14 +322,14 @@ public class Herold {
 			script = mgr.parseScript(profileFile);
 			logger.info(String.format("Using profile file %s.",
 					profileFile.getAbsolutePath()));
-
+			
 			if (logger.isTraceEnabled()) {
 				StringWriter buffer = new StringWriter();
 				mgr.writeScript(script, buffer);
 				logger.trace("Script: " + buffer.toString());
 			}
 
-			File outFile = converter.processCommandLineOptions(options, script);
+			File outFile = processCommandLineOptions(options, script);
 
 			if (outFile != null) {
 				logger.info(String.format("Output file is %s.",
@@ -143,7 +347,7 @@ public class Herold {
 
 			out = createOutputStream();
 
-			converter.convert(in, out, script);
+			convert(in, out, optOut.getValue(), script);
 
 		} catch (OptionException oops) {
 
@@ -188,30 +392,16 @@ public class Herold {
 		exit(exitCode);
 	}
 
-	private static OutputStream createOutputStream()
-			throws FileNotFoundException {
+	private void exit(int exitCode) {
 
-		OutputStream out;
+		this.exitCode = exitCode;
 
-		if (optOut.isUnset()) {
-
-			out = System.out;
-
-		} else {
-
-			File outDir = optOut.getValue().getParentFile();
-
-			if (outDir != null && outDir.exists() == false) {
-				outDir.mkdir();
-			}
-
-			out = new FileOutputStream(optOut.getValue());
+		if (systemExitEnabled == true) {
+			System.exit(exitCode);
 		}
-
-		return out;
 	}
 
-	private static File findProfile(StringOption optProfile)
+	private File findProfile(StringOption optProfile)
 			throws FileNotFoundException {
 
 		String profileName = "default.her";
@@ -266,122 +456,15 @@ public class Herold {
 				"Profile %s does not exist!", profileName));
 	}
 
-	private static void exit(int exitCode) {
-
-		Herold.exitCode = exitCode;
-
-		if (systemExitEnabled == true) {
-			System.exit(exitCode);
-		}
+	public int getExitCode() {
+		return exitCode;
 	}
 
-	private static OptionList createOptionList(String[] args) {
-
-		Option<?> option;
-		SelectOption selopt;
-		BooleanOption bopt;
-		StringOption sopt;
-
-		OptionList options = new OptionList(args);
-
-		// docbook-add-index
-		bopt = new BooleanOption(DbtConstants.SECTION_DOCBOOK.toLowerCase()
-				+ "-" + DbtConstants.PARAM_ADD_INDEX, "x");
-		bopt.setValue(false);
-		options.add(bopt);
-
-		// docbook-encoding
-		sopt = new StringOption(DbtConstants.SECTION_DOCBOOK.toLowerCase()
-				+ "-" + DbtConstants.PARAM_ENCODING, "d");
-		sopt.setDefault("UTF-8");
-		options.add(sopt);
-
-		// decompose-tables
-		bopt = new BooleanOption(DbtConstants.SECTION_DOCBOOK.toLowerCase()
-				+ "-" + DbtConstants.PARAM_DECOMPOSE_TABLES, "T");
-		bopt.setDefault(false);
-		options.add(bopt);
-
-		// document-element
-		selopt = new SelectOption(DbtConstants.SECTION_DOCBOOK.toLowerCase()
-				+ "-" + DbtConstants.PARAM_DOCUMENT_ELEMENT, "r");
-
-		String[] optv2 = { "article", "book", "reference", "part", "chapter",
-				"section" };
-		selopt.setList(optv2);
-		selopt.setDefault("article");
-		options.add(selopt);
-
-		// title
-		sopt = new StringOption(DbtConstants.SECTION_DOCBOOK.toLowerCase()
-				+ "-" + DbtConstants.PARAM_TITLE, "t");
-		sopt.setDefault("http://www.dbdoclet.org/herold");
-		options.add(sopt);
-
-		// source-encoding
-		sopt = new StringOption(DbtConstants.SECTION_HTML.toLowerCase() + "-"
-				+ DbtConstants.PARAM_HTML_SOURCE_ENCODING, "s");
-		sopt.setDefault("UTF-8");
-		options.add(sopt);
-
-		// in
-		optIn = new FileOption("in", "i");
-		optIn.isExisting(true);
-		options.add(optIn);
-
-		// loglevel
-		SelectOption optLogLevel = new SelectOption("logging-level", "l");
-		optLogLevel.setList(new String[] { "debug5", "debug4", "debug3",
-				"debug2", "debug", "info", "warn", "error", "fatal" });
-		optLogLevel.setDefault("error");
-		options.add(optLogLevel);
-
-		// out
-		optOut = new FileOption("out", "o");
-		options.add(optOut);
-
-		StringOption optProfile = new StringOption("profile", "p");
-		options.add(optProfile);
-
-		// verbose
-		option = new BooleanOption("verbose", "v");
-		options.add(option);
-
-		return options;
+	public boolean isCanceled() {
+		return false;
 	}
 
-	private static String getVersion() {
-
-		Package p = Herold.class.getPackage();
-		return p.getImplementationVersion();
-	}
-
-	private static void printLicense() throws IOException {
-
-		URL url = ResourceServices
-				.getResourceAsUrl("/org/dbdoclet/herold/COPYING");
-
-		if (url != null) {
-			BufferedReader reader = new BufferedReader(new InputStreamReader(
-					url.openStream()));
-			String line = reader.readLine();
-
-			while (line != null) {
-				println(line);
-				line = reader.readLine();
-			}
-
-			reader.close();
-		} else {
-			logger.fatal("Can't find resource for license!");
-		}
-	}
-
-	private static void println(String str) {
-		System.out.println(str);
-	}
-
-	private static void printUsage() {
+	private void printUsage() {
 
 		try {
 
@@ -396,60 +479,40 @@ public class Herold {
 		}
 	}
 
-	private static void printVersion() {
-
-		println("herold version \"" + getVersion() + "\"");
-	}
-
-	public void convert(InputStream in, OutputStream out, Script script)
-			throws TrafoException {
-
-		DocBookTransformer trafo = new DocBookTransformer();
-
-		if (verbose == true) {
-			trafo.addProgressListener(new ConsoleProgressListener(false));
-		}
-
-		trafo.setScript(script);
-		trafo.convert(in, out);
-
-		if (verbose == true) {
-			System.out.println();
-		}
-	}
-
-	public boolean isCanceled() {
-		return false;
-	}
-
 	public File processCommandLineOptions(OptionList options, Script script) {
 
+		Namespace namespace = script.getNamespace();
+		Section section = null;
+		
 		for (Option<?> option : options) {
 
 			String name = option.getLongName();
 
-			if (name.startsWith(DbtConstants.SECTION_DOCBOOK.toLowerCase())) {
-				script.selectSection(DbtConstants.SECTION_DOCBOOK);
+			if (name.startsWith(TrafoConstants.SECTION_DOCBOOK.toLowerCase())) {
+				
+				section = namespace.findOrCreateSection(TrafoConstants.SECTION_DOCBOOK);
 				name = StringServices.cutPrefix(name,
-						DbtConstants.SECTION_DOCBOOK.toLowerCase());
+						TrafoConstants.SECTION_DOCBOOK.toLowerCase());
 
-			} else if (name.startsWith(DbtConstants.SECTION_HTML.toLowerCase())) {
-				script.selectSection(DbtConstants.SECTION_HTML);
+			} else if (name.startsWith(TrafoConstants.SECTION_HTML
+					.toLowerCase())) {
+
+				section = namespace.findOrCreateSection(TrafoConstants.SECTION_HTML);
 				name = StringServices.cutPrefix(name,
-						DbtConstants.SECTION_HTML.toLowerCase());
+						TrafoConstants.SECTION_HTML.toLowerCase());
 			}
 
 			name = StringServices.cutPrefix(name, "-");
 
-			if (option.isUnset() == false) {
+			if (section != null && option.isUnset() == false) {
 				switch (option.getType()) {
 
 				case BOOLEAN:
-					script.setBoolParameter(name, (Boolean) option.getValue());
+					section.setParam(new BooleanParam(name, (Boolean) option.getValue()));
 					break;
 
 				default:
-					script.setTextParameter(name, option.getValue().toString());
+					section.setParam(new TextParam(name, option.getValue().toString()));
 					break;
 				}
 			}
@@ -459,6 +522,16 @@ public class Herold {
 
 		File outFile;
 
+		if (optOut == null) {
+			throw new IllegalStateException(
+					"The field optOut must not be null!");
+		}
+
+		if (optIn == null) {
+			throw new IllegalStateException(
+					"The field optIn must not be null!");
+		}
+
 		if (optOut.isUnset() && optIn.isUnset() == false) {
 
 			String outFileName = FileServices.getFileBase(optIn.getValue())
@@ -473,23 +546,11 @@ public class Herold {
 		return outFile;
 	}
 
+	public void setSystemExitEnabled(boolean systemExitEnabled) {
+		this.systemExitEnabled = systemExitEnabled;
+	}
+
 	public void setVerbose(boolean verbose) {
 		this.verbose = verbose;
 	}
-
-	public void convert(File htmlFile, File xmlFile) throws TrafoException,
-			FileNotFoundException {
-
-		Script script = new Script();
-		convert(new FileInputStream(htmlFile), new FileOutputStream(xmlFile),
-				script);
-	}
-
-	public static void setSystemExitEnabled(boolean systemExitEnabled) {
-		Herold.systemExitEnabled = systemExitEnabled;
-	}
-
-	public static int getExitCode() {
-		return exitCode;
-	}
 }
