@@ -1,18 +1,20 @@
 /* 
- * $Id$
+ * Copyright (C) 2001-2013 Michael Fuchs
  *
- * ### Copyright (C) 2005 Michael Fuchs ###
- * ### All Rights Reserved.             ###
- *
- * Author: Michael Fuchs
- * E-Mail: michael.fuchs@unico-group.com
- * URL:    http://www.michael-a-fuchs.de
- *
- * RCS Information
- * Author..........: $Author$
- * Date............: $Date$
- * Revision........: $Revision$
- * State...........: $State$
+ * This file is part of herold.
+ * 
+ * herold is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * herold is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with herold.  If not, see <http://www.gnu.org/licenses/>.  
  */
 package org.dbdoclet.option;
 
@@ -89,20 +91,9 @@ public class OptionList implements Iterable<Option<?>> {
 	 */
 	public OptionList(String[][] cmdline) {
 
-		ArrayList<String> list = new ArrayList<String>();
-
-		for (int i = 0; i < cmdline.length; i++) {
-			for (int j = 0; j < cmdline[i].length; j++) {
-				list.add(cmdline[i][j]);
-			}
-		}
-
-		String[] args = new String[list.size()];
-
-		int index = 0;
-		for (Iterator<String> i = list.iterator(); i.hasNext();) {
-			args[index++] = i.next();
-		}
+		String[] args = Arrays.stream(cmdline)
+				.flatMap(arr -> Arrays.stream(arr))
+				.toArray(String[]::new);
 
 		init(args, 0);
 	}
@@ -164,7 +155,7 @@ public class OptionList implements Iterable<Option<?>> {
 			error = "";
 		}
 
-		return error;
+		return String.join(" ", args) + "\n\n" + error;
 	}
 
 	/**
@@ -316,13 +307,7 @@ public class OptionList implements Iterable<Option<?>> {
 		}
 
 		optList.add(option);
-
-		String[] names = option.getNames();
-
-		for (int i = 0; i < names.length; i++) {
-			optMap.put(names[i], option);
-		}
-
+		Arrays.stream(option.getNames()).forEach(n -> optMap.put(n, option));
 	}
 
 	/**
@@ -338,13 +323,7 @@ public class OptionList implements Iterable<Option<?>> {
 			return new String[0];
 		}
 
-		String[] array = new String[remainingArgs.size()];
-
-		for (int i = 0; i < remainingArgs.size(); i++) {
-			array[i] = remainingArgs.get(i);
-		}
-
-		return array;
+		return remainingArgs.stream().toArray(String[]::new);
 	}
 
 	/**
@@ -460,6 +439,7 @@ public class OptionList implements Iterable<Option<?>> {
 
 		String buffer = "";
 
+		// props.forEach(Property p -> System.out.println());
 		for (Iterator<Object> i = props.keySet().iterator(); i.hasNext();) {
 
 			obj = i.next();
@@ -511,8 +491,8 @@ public class OptionList implements Iterable<Option<?>> {
 
 				if (option == null) {
 
-					buffer += MessageFormat.format(ResourceServices
-							.getString(res, "C_ERROR_PROPERTY_UNKNOWN"), name)
+					buffer += MessageFormat.format(ResourceServices.getString(
+							res, "C_ERROR_PROPERTY_UNKNOWN"), name)
 							+ "\n";
 					continue;
 				}
@@ -588,10 +568,8 @@ public class OptionList implements Iterable<Option<?>> {
 			index = searchArgs(argList, option);
 
 			if (index == -1 && option.isRequired()) {
-				buffer += MessageFormat
-						.format(ResourceServices.getString(res,
-								"C_ERROR_OPTION_REQUIRED"), option
-								.getUniqueName())
+				buffer += MessageFormat.format(ResourceServices.getString(res,
+						"C_ERROR_OPTION_REQUIRED"), option.getUniqueName())
 						+ "\n";
 				continue;
 			}
@@ -767,9 +745,8 @@ public class OptionList implements Iterable<Option<?>> {
 
 				if (option.isUnique() && index != -1 && counter == 0) {
 
-					buffer += MessageFormat.format(ResourceServices
-							.getString(res,
-									"C_ERROR_OPTION_WAS_FOUND_MORE_THAN_ONCE"),
+					buffer += MessageFormat.format(ResourceServices.getString(
+							res, "C_ERROR_OPTION_WAS_FOUND_MORE_THAN_ONCE"),
 							option.getUniqueName())
 							+ "\n";
 				}
@@ -1037,15 +1014,15 @@ public class OptionList implements Iterable<Option<?>> {
 	public Option<String> getTextOption(String name) {
 
 		Option<?> option = getOption(name);
-		
+
 		if (option instanceof SelectOption) {
 			return (SelectOption) option;
 		}
-		
+
 		if (option instanceof StringOption) {
 			return (StringOption) option;
 		}
-		
+
 		return null;
 	}
 
