@@ -1,3 +1,21 @@
+/* 
+ * Copyright (C) 2001-2013 Michael Fuchs
+ *
+ * This file is part of herold.
+ * 
+ * herold is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * herold is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with herold.  If not, see <http://www.gnu.org/licenses/>.  
+ */
 package org.dbdoclet.trafo;
 
 import java.io.File;
@@ -6,20 +24,25 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.PrintWriter;
 import java.io.Writer;
-import java.util.LinkedHashMap;
 import java.util.List;
 
-import org.antlr.runtime.ANTLRFileStream;
-import org.antlr.runtime.ANTLRInputStream;
-import org.antlr.runtime.ANTLRStringStream;
-import org.antlr.runtime.CommonTokenStream;
-import org.antlr.runtime.RecognitionException;
-import org.antlr.runtime.tree.CommonTreeNodeStream;
+import org.antlr.v4.runtime.ANTLRFileStream;
+import org.antlr.v4.runtime.ANTLRInputStream;
+import org.antlr.v4.runtime.CommonTokenStream;
+import org.antlr.v4.runtime.RecognitionException;
+import org.antlr.v4.runtime.tree.ParseTree;
+import org.antlr.v4.runtime.tree.ParseTreeWalker;
 import org.dbdoclet.trafo.param.Param;
+import org.dbdoclet.trafo.script.AttributeRule;
+import org.dbdoclet.trafo.script.Namespace;
+import org.dbdoclet.trafo.script.NodeRule;
 import org.dbdoclet.trafo.script.Script;
+import org.dbdoclet.trafo.script.Section;
 import org.dbdoclet.trafo.script.parser.TrafoScriptLexer;
+import org.dbdoclet.trafo.script.parser.TrafoScriptListener;
+import org.dbdoclet.trafo.script.parser.TrafoScriptListenerImpl;
 import org.dbdoclet.trafo.script.parser.TrafoScriptParser;
-import org.dbdoclet.trafo.script.parser.TrafoScriptWalker;
+import org.dbdoclet.trafo.script.parser.TrafoScriptParserErrorListener;
 
 public class TrafoScriptManager {
 
@@ -60,7 +83,7 @@ public class TrafoScriptManager {
 
 		try {
 			TrafoScriptLexer lex = new TrafoScriptLexer(new ANTLRInputStream(
-					instr, "UTF8"));
+					instr));
 			parseScript(script, lex, namespace);
 		} catch (Exception oops) {
 			throw new TrafoException(oops);
@@ -79,8 +102,7 @@ public class TrafoScriptManager {
 	public Script parseScript(String scriptBuffer) throws IOException,
 			RecognitionException, TrafoException {
 
-		TrafoScriptLexer lex = new TrafoScriptLexer(new ANTLRStringStream(
-				scriptBuffer));
+		TrafoScriptLexer lex = new TrafoScriptLexer(new ANTLRInputStream(scriptBuffer));
 
 		Script script = new Script();
 		parseScript(script, lex, "");
@@ -93,19 +115,21 @@ public class TrafoScriptManager {
 		CommonTokenStream tokens = new CommonTokenStream(lex);
 
 		TrafoScriptParser parser = new TrafoScriptParser(tokens);
-		TrafoScriptParser.parse_return r = parser.parse();
+		parser.removeErrorListeners();
+		TrafoScriptParserErrorListener errorListener = new TrafoScriptParserErrorListener();
+		parser.addErrorListener(errorListener);
+		ParseTree parserTree  = parser.parse();
 
-		List<String> errors = parser.getErrors();
+		List<String> errors = errorListener.getErrors();
 
 		if (errors != null && errors.size() > 0) {
 			throw new TrafoException(errors.get(0));
 		}
 
-		CommonTreeNodeStream nodes = new CommonTreeNodeStream(r.getTree());
-
-		TrafoScriptWalker walker = new TrafoScriptWalker(nodes);
-		walker.parse(script, namespace);
-
+		ParseTreeWalker walker = new ParseTreeWalker();
+		TrafoScriptListener listener = new TrafoScriptListenerImpl(script, namespace);
+		walker.walk(listener, parserTree);
+		
 		return script;
 	}
 
@@ -113,7 +137,7 @@ public class TrafoScriptManager {
 			throws TrafoException {
 
 		try {
-			TrafoScriptLexer lex = new TrafoScriptLexer(new ANTLRStringStream(
+			TrafoScriptLexer lex = new TrafoScriptLexer(new ANTLRInputStream(
 					scriptBuffer));
 			parseScript(script, lex, "");
 		} catch (Exception oops) {
@@ -134,7 +158,7 @@ public class TrafoScriptManager {
 		writeScript(Script.DEFAULT_NAMESPACE, script, writer);
 	}
 
-	public void writeScript(String namespace, Script script, Writer writer)
+	public void writeScript(String namespaceName, Script script, Writer writer)
 			throws IOException {
 
 		PrintWriter printWriter = null;
@@ -145,7 +169,7 @@ public class TrafoScriptManager {
 
 			String transformationName = "trafo";
 
-			Param<?> sysparam = script.getSystemParameter(namespace,
+			Param<?> sysparam = script.getSystemParameter(namespaceName,
 					Script.SYSPARAM_TRANSFORMATION_NAME);
 
 			if (sysparam != null) {
@@ -155,30 +179,53 @@ public class TrafoScriptManager {
 			printWriter.println(String.format("transformation %s;",
 					transformationName));
 
-			LinkedHashMap<String, LinkedHashMap<String, Param<?>>> sectionMap = script
-					.getSectionMap(namespace);
+			Namespace namespace = script.getNamespace(namespaceName);
+			
+			if (namespace == null) {
+				return;
+			}
+			
+			for (Section section: namespace.getSections()) {
 
-			if (sectionMap != null) {
-
-				for (String sectionName : sectionMap.keySet()) {
-
-					if (sectionName.startsWith("SYS ")) {
+				if (section.getName().startsWith("SYS ")) {
 						continue;
-					}
+				}
 
-					printWriter.println(String.format("%nsection %s {",
-							sectionName));
+				printWriter.println(String.format("%nsection %s {",
+							section.getName()));
 
-					LinkedHashMap<String, Param<?>> paramMap = sectionMap
-							.get(sectionName);
-
-					for (Param<?> param : paramMap.values()) {
+				for (Param<?> param : section.getParameters()) {
 						printWriter.println(String.format("\t%s;",
 								param.toString()));
-					}
-
-					printWriter.println("}");
 				}
+
+				printWriter.println("}");				
+			}
+
+			for (NodeRule nodeRule : namespace.getNodeRules()) {
+
+				printWriter.println(String.format("%nnode %s {",
+							nodeRule.getName()));
+
+				for (Param<?> param : nodeRule.getParameters()) {
+						printWriter.println(String.format("\t%s;",
+								param.toString()));
+				}
+
+				printWriter.println("}");				
+			}
+
+			for (AttributeRule attributeRule : namespace.getAttributeRules()) {
+
+				printWriter.println(String.format("%nattribute %s {",
+							attributeRule.getName()));
+
+				for (Param<?> param : attributeRule.getParameters()) {
+						printWriter.println(String.format("\t%s;",
+								param.toString()));
+				}
+
+				printWriter.println("}");				
 			}
 
 		} finally {
