@@ -1,13 +1,20 @@
 /* 
-+ * $Id$
+ * Copyright (C) 2001-2013 Michael Fuchs
  *
- * ### Copyright (C) 2003-2007 Michael Fuchs ###
- * ### All Rights
- Reserved.             ###
- *
- * Author: Michael Fuchs
- * E-Mail: michael.fuchs@unico-group.com
- * URL:    http://www.michael-a-fuchs.de
+ * This file is part of herold.
+ * 
+ * herold is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * herold is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with herold.  If not, see <http://www.gnu.org/licenses/>.  
  */
 package org.dbdoclet.xiphias;
 
@@ -18,6 +25,7 @@ import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.FileReader;
 import java.io.IOException;
+import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
 import java.io.StringReader;
 import java.io.UnsupportedEncodingException;
@@ -77,11 +85,6 @@ import org.xml.sax.helpers.XMLReaderFactory;
  */
 public class XmlServices {
 
-	public enum HyphenationChar {
-		SOFT_HYPHEN, ZERO_WIDTH_SPACE
-	}
-
-	public static HyphenationChar hyphenationChar = HyphenationChar.ZERO_WIDTH_SPACE;
 	public static final String test = "&nbsp;";
 	private static final Pattern encPattern = Pattern
 			.compile("^(?i).*<\\?xml\\s*.*encoding=['\"](.*)['\"].*\\?>.*$");
@@ -548,64 +551,6 @@ public class XmlServices {
 		return true;
 	}
 
-	public static String makeWrapable(String buffer, String token) {
-		return makeWrapable(buffer, token, hyphenationChar);
-	}
-
-	public static String makeWrapable(String buffer, String token,
-			HyphenationChar hyphenChar) {
-
-		if (buffer == null || buffer.length() == 0) {
-			return buffer;
-		}
-
-		if (token == null || token.length() == 0) {
-			return buffer;
-		}
-
-		String[] tokens = { token };
-
-		return makeWrapable(buffer, tokens, hyphenChar);
-	}
-
-	public static String makeWrapable(String buffer, String[] tokens) {
-		return makeWrapable(buffer, tokens, hyphenationChar);
-	}
-
-	public static String makeWrapable(String buffer, String[] tokens,
-			HyphenationChar hyphenChar) {
-
-		if (buffer == null || buffer.length() == 0) {
-			return buffer;
-		}
-
-		if (tokens == null || tokens.length == 0) {
-			return buffer;
-		}
-
-		String wrapable = buffer;
-		String token;
-
-		for (int i = 0; i < tokens.length; i++) {
-
-			token = tokens[i];
-
-			if (token == null || token.length() == 0) {
-				continue;
-			}
-
-			if (hyphenChar == HyphenationChar.SOFT_HYPHEN) {
-				wrapable = StringServices.replace(wrapable, token, token
-						+ "&#x00ad;");
-			} else {
-				wrapable = StringServices.replace(wrapable, token, token
-						+ "&#x200b;");
-			}
-		}
-
-		return wrapable;
-	}
-
 	public static String normalizeText(String text) {
 
 		if (text == null) {
@@ -627,7 +572,17 @@ public class XmlServices {
 		return parse(source, validate, null);
 	}
 
-	public static Document parse(File source, boolean validate, URL schemaUrl)
+	public static Document parse(File file, boolean validate, URL schemaUrl)
+			throws IOException, SAXException, ParserConfigurationException {
+
+		String encoding = getEncoding(file);
+		InputStreamReader reader = new InputStreamReader(new FileInputStream(file), encoding);
+		InputSource inputSource = new InputSource(reader);
+		inputSource.setSystemId(file.getCanonicalPath());
+		return parse(inputSource, validate, null);
+	}
+	
+	public static Document parse(InputSource source, boolean validate, URL schemaUrl)
 			throws IOException, SAXException, ParserConfigurationException {
 
 		if (source == null) {
@@ -662,7 +617,7 @@ public class XmlServices {
 			parser.setEntityResolver(new CatalogResolver());
 		}
 
-		XmlValidationResult result = new XmlValidationResult(source,
+		XmlValidationResult result = new XmlValidationResult(new File(source.getSystemId()),
 				Locale.getDefault());
 		parser.setErrorHandler(result);
 
@@ -739,7 +694,8 @@ public class XmlServices {
 
 		Document doc = parse(file, false, schemaUrl);
 		NodeSerializer serializer = new NodeSerializer();
-		serializer.write(doc, file, to);
+		serializer.setEncoding(to);
+		serializer.write(doc, file);
 	}
 
 	public static String replaceAmpersand(String text, boolean isHtml) {
@@ -1544,11 +1500,15 @@ public class XmlServices {
 
 			Schema schema = SchemaFactory.newInstance(
 					XMLConstants.W3C_XML_SCHEMA_NS_URI).newSchema(schemaUrl);
+			
 			factory = DocumentBuilderFactory.newInstance();
 			factory.setNamespaceAware(true);
 			factory.setSchema(schema);
 			factory.setXIncludeAware(true);
+
 			parser = factory.newDocumentBuilder();
+			parser.setEntityResolver(new CatalogResolver());
+			
 
 		} else {
 
