@@ -1,14 +1,28 @@
 /* 
- * ### Copyright (C) 2003-2009 Michael Fuchs ###
- * ### All Rights Reserved.             ###
+ * Copyright (C) 2001-2013 Michael Fuchs
  *
- * Author: Michael Fuchs
- * E-Mail: michael.fuchs@dbdoclet.org
- * URL:    http://www.michael-a-fuchs.de
+ * This file is part of herold.
+ * 
+ * herold is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * herold is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with herold.  If not, see <http://www.gnu.org/licenses/>.  
  */
 package org.dbdoclet.xiphias.dom;
 
+import java.util.HashMap;
+
+import org.dbdoclet.Sfv;
 import org.dbdoclet.xiphias.W3cServices;
+import org.dbdoclet.xiphias.XmlConstants;
 import org.w3c.dom.Attr;
 import org.w3c.dom.CDATASection;
 import org.w3c.dom.Comment;
@@ -27,11 +41,12 @@ import org.w3c.dom.Text;
 
 public class DocumentImpl extends NodeImpl implements Document {
 
+	private DocumentType docType;
+	private ElementImpl documentElement;
+	private HashMap<String, String> namespacePrefixMap = new HashMap<>();
 	private String xmlEncoding = "UTF-8";
 	private String xmlVersion = "1.0";
-	private ElementImpl documentElement;
-	private DocumentType docType;
-
+	
 	public DocumentImpl() {
 
 		super("#document", null);
@@ -39,6 +54,8 @@ public class DocumentImpl extends NodeImpl implements Document {
 
 		setXmlEncoding("UTF-8");
 		setXmlVersion("1.0");
+		
+		namespacePrefixMap.put(XmlConstants.NAMESPACE_XML, "xml");
 	}
 
 	@Override
@@ -48,13 +65,24 @@ public class DocumentImpl extends NodeImpl implements Document {
 
 	@Override
 	public Attr createAttribute(String name) throws DOMException {
-		throw new IllegalStateException("Not yet implemented");
+		
+		AttrImpl attr = new AttrImpl();
+		attr.setNodeName(name);
+		return attr;
+		
 	}
 
 	@Override
-	public Attr createAttributeNS(String namespaceURI, String qualifiedName)
+	public Attr createAttributeNS(String namespaceUri, String qualifiedName)
 			throws DOMException {
-		throw new IllegalStateException("Not yet implemented");
+
+		String prefix = manageNamespace(namespaceUri, qualifiedName);
+		
+		AttrImpl attr = new AttrImpl();
+		attr.setPrefix(prefix);
+		attr.setNamespaceURI(namespaceUri);
+		attr.setNodeName(qualifiedName);
+		return attr;
 	}
 
 	@Override
@@ -82,9 +110,16 @@ public class DocumentImpl extends NodeImpl implements Document {
 	}
 
 	@Override
-	public Element createElementNS(String namespaceURI, String qualifiedName)
+	public Element createElementNS(String namespaceUri, String qualifiedName)
 			throws DOMException {
-		throw new IllegalStateException("Not yet implemented");
+
+		String prefix = manageNamespace(namespaceUri, qualifiedName);
+		
+		ElementImpl elem = new ElementImpl();
+		elem.setNamespaceURI(namespaceUri);
+		elem.setPrefix(prefix);
+		elem.setNodeName(qualifiedName);
+		return elem;
 	}
 
 	@Override
@@ -103,12 +138,13 @@ public class DocumentImpl extends NodeImpl implements Document {
 	public Text createTextNode(String data) {
 
 		TextImpl text = new TextImpl(data);
+		text.setDocument(this);
 		return text;
 	}
 
 	public String createXmlDeclaration() {
 		return "<?xml version=\"" + xmlVersion + "\" encoding=\"" + xmlEncoding
-				+ "\"?>\n";
+				+ "\"?>" + Sfv.LSEP;
 	}
 
 	@Override
@@ -117,7 +153,12 @@ public class DocumentImpl extends NodeImpl implements Document {
 	}
 
 	@Override
-	public ElementImpl getDocumentElement() {
+	public Element getDocumentElement() {
+
+		if (documentElement == null) {
+			return (Element) getFirstChildElement();
+		}
+
 		return documentElement;
 	}
 
@@ -203,14 +244,15 @@ public class DocumentImpl extends NodeImpl implements Document {
 		throw new IllegalStateException("Not yet implemented");
 	}
 
-	public void setDocumentElement(ElementImpl documentElement) {
-		this.documentElement = documentElement;
-	}
-
 	public void setDoctype(DocumentType docType) {
 		this.docType = docType;
 	}
 
+	public void setDocumentElement(ElementImpl documentElement) {
+		this.documentElement = documentElement;
+
+	}
+
 	@Override
 	public void setDocumentURI(String documentURI) {
 		throw new IllegalStateException("Not yet implemented");
@@ -238,4 +280,30 @@ public class DocumentImpl extends NodeImpl implements Document {
 		this.xmlVersion = xmlVersion;
 
 	}
+
+	private String manageNamespace(String namespaceUri, String qualifiedName) {
+		
+		String prefix = null;
+
+		if (qualifiedName.contains(":")) {
+			String[] tokens = qualifiedName.split(":");
+			prefix = tokens[0];
+		}
+
+		if (namespaceUri == null && prefix != null) {
+			throw new DOMException(DOMException.NAMESPACE_ERR, String.format(
+					"No namespace for prefix %s defined", prefix));
+		}
+
+		if (namespaceUri != null && prefix != null) {
+			
+			String value = namespacePrefixMap.get(namespaceUri);
+			
+			if (value == null) {
+				namespacePrefixMap.put(namespaceUri, prefix);
+			}
+		}
+		
+		return prefix;
+	}
 }
