@@ -1,24 +1,39 @@
 /* 
- * ### Copyright (C) 2001-2007 Michael Fuchs ###
- * ### All Rights Reserved.             ###
+ * Copyright (C) 2001-2013 Michael Fuchs
  *
- * Author: Michael Fuchs
- * E-Mail: michael.fuchs@unico-group.com
- * URL:    http://www.michael-a-fuchs.de
+ * This file is part of herold.
+ * 
+ * herold is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * herold is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with herold.  If not, see <http://www.gnu.org/licenses/>.  
  */
 package org.dbdoclet.xiphias.dom;
 
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
+import java.util.stream.Collectors;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.dbdoclet.service.StringServices;
 import org.dbdoclet.xiphias.XmlServices;
+import org.dbdoclet.xiphias.annotation.Annotation;
+import org.w3c.dom.Attr;
 import org.w3c.dom.DOMException;
 import org.w3c.dom.Document;
+import org.w3c.dom.DocumentFragment;
 import org.w3c.dom.Element;
 import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
@@ -34,19 +49,37 @@ import org.w3c.dom.UserDataHandler;
  */
 public class NodeImpl implements Node {
 
-	public static final int FORMAT_BLOCK = 3;
 
+	public static final int FORMAT_BLOCK = 3;
 	public static final int FORMAT_CONTENT = 2;
 	public static final int FORMAT_INLINE = 1;
 	public static final int HTML = 3;
-	public static final int SGML = 1;
-	public static final int XML = 2;
-	public static final short XML_DECLARATION = 23;
-	private static int flavour = XML;
-
 	private static Log logger = LogFactory.getLog(NodeImpl.class);
 	private static final String LSEP = System.getProperty("line.separator");
 
+	public static final int SGML = 1;
+	public static final int XML = 2;
+	private static int flavour = XML;
+	public static final short XML_DECLARATION = 23;
+
+	private NodeListImpl childNodes = new NodeListImpl();
+	private int column;
+	private DocumentImpl document;
+	protected boolean isCaseInsensitive = false;
+	private boolean isEmpty = false;
+	private boolean isMute = false;
+	private boolean isRawData = false;
+	private int line;
+	private String namespaceUri;
+	private boolean needsPadding = false;
+	private String nodeName = "node";
+	private short nodeType = ELEMENT_NODE;
+	private String nodeValue;
+	private NodeImpl parent = null;
+	private String prefix;
+	private HashMap<String, Object> userDataMap;
+	private ArrayList<Annotation> annotationList;
+
 	/**
 	 * The method <code>findParent</code> tries to find the nearest parent,
 	 * which is of type <code>parentClass</code>.
@@ -161,7 +194,12 @@ public class NodeImpl implements Node {
 		if (node != null) {
 
 			if (node instanceof Document) {
-				node = ((Document) node).getDocumentElement();
+				
+				Document doc = (Document) node;
+				
+				if (doc.getDocumentElement() != null) {
+					node = doc.getDocumentElement();
+				}
 			}
 
 			if (visitor != null) {
@@ -176,29 +214,6 @@ public class NodeImpl implements Node {
 		}
 	}
 
-	private NodeListImpl childNodes = new NodeListImpl();
-	private DocumentImpl document;
-	private boolean isEmpty = false;
-	private boolean isMute = false;
-	private boolean isRawData = false;
-	private String namespaceUri;
-	private boolean needsPadding = false;
-	private String nodeName = "node";
-	private short nodeType = ELEMENT_NODE;
-	private String nodeValue;
-
-	private NodeImpl parent = null;
-
-	private TransformInstruction transformInstruction;
-
-	private HashMap<String, Object> userDataMap;
-
-	private int column;
-
-	private int line;
-
-	protected boolean isCaseInsensitive = false;
-
 	public NodeImpl() {
 		super();
 	}
@@ -220,6 +235,18 @@ public class NodeImpl implements Node {
 		this.parent = parent;
 	}
 
+	public void addAnnotation(Annotation annotation) {
+		
+		if (annotation != null) {
+			
+			if (annotationList == null) {
+				annotationList = new ArrayList<>();
+			}
+			
+			annotationList.add(annotation);
+		}
+	}
+
 	@Override
 	public Node appendChild(Node newChild) throws DOMException {
 
@@ -251,7 +278,19 @@ public class NodeImpl implements Node {
 		}
 
 		node.setParentNode(this);
-		childNodes.add(node);
+		
+		if (node instanceof DocumentFragment) {
+			
+			NodeList childList = node.getChildNodes();
+			
+			for (int i=0; i<childList.getLength(); i++) {
+				childNodes.add((NodeImpl) childList.item(i));
+			}
+			
+		} else {
+			
+			childNodes.add(node);
+		}
 
 		logger.debug("Appended child " + node + " to " + this + ".");
 
@@ -295,23 +334,6 @@ public class NodeImpl implements Node {
 		return 0;
 	}
 
-	public Text findFirstText() {
-
-		Text firstText = getFirstText();
-		if (firstText != null) {
-			return firstText;
-		}
-
-		for (Element element : getChildElementList()) {
-			firstText = ((ElementImpl) element).findFirstText();
-			if (firstText != null) {
-				return firstText;
-			}
-		}
-
-		return null;
-	}
-
 	public NodeImpl findChildElement(String tagName) {
 
 		if (tagName == null) {
@@ -342,6 +364,23 @@ public class NodeImpl implements Node {
 		return null;
 	}
 
+	public Text findFirstText() {
+
+		Text firstText = getFirstText();
+		if (firstText != null) {
+			return firstText;
+		}
+
+		for (Element element : getChildElementList()) {
+			firstText = ((ElementImpl) element).findFirstText();
+			if (firstText != null) {
+				return firstText;
+			}
+		}
+
+		return null;
+	}
+
 	public NodeImpl findParent(Class<?> parentClass) {
 
 		Node parent = this.getParentNode();
@@ -357,6 +396,15 @@ public class NodeImpl implements Node {
 		return null;
 	}
 
+	public Map<String, Attr> getAttributesAsMap() {
+
+		if (this instanceof ElementImpl) {
+			return this.getAttributesAsMap();
+		}
+
+		return null;
+	}
+
 	@Override
 	public String getBaseURI() {
 		System.err.println("Not yet implemented");
@@ -401,6 +449,21 @@ public class NodeImpl implements Node {
 	}
 
 	public DocumentImpl getDocument() {
+		
+		if (document == null) {
+		
+			Node parent = getParentNode();
+			
+			while (parent != null && document == null) {
+				document = (DocumentImpl) parent.getOwnerDocument();
+				parent = parent.getParentNode();
+			}
+		}
+		
+		if (document == null) {
+			document = new DocumentImpl();
+		}
+		
 		return document;
 	}
 
@@ -457,7 +520,7 @@ public class NodeImpl implements Node {
 	 * 
 	 * @return The element or null, if element child can be found.
 	 */
-	public Element getFirstElement() {
+	public Element getFirstChildElement() {
 
 		if ((childNodes == null) || (childNodes.size() < 1)) {
 			return null;
@@ -618,8 +681,7 @@ public class NodeImpl implements Node {
 
 	@Override
 	public Document getOwnerDocument() {
-		System.err.println("Not yet implemented");
-		return null;
+		return getDocument();
 	}
 
 	@Override
@@ -630,6 +692,10 @@ public class NodeImpl implements Node {
 	@Override
 	public String getPrefix() {
 
+		if (prefix != null) {
+			return prefix;
+			
+		}
 		if (this instanceof ElementImpl || this instanceof AttrImpl) {
 
 			String qualifiedName = getNodeName();
@@ -699,15 +765,6 @@ public class NodeImpl implements Node {
 		return NodeImpl.getTextContent(this);
 	}
 
-	public Map<String, AttrImpl> getTrafoAttributes() {
-
-		if (this instanceof ElementImpl) {
-			return this.getTrafoAttributes();
-		}
-
-		return null;
-	}
-
 	public NodeListImpl getTrafoChildNodes() {
 
 		if (childNodes == null) {
@@ -721,10 +778,6 @@ public class NodeImpl implements Node {
 		return parent;
 	}
 
-	public TransformInstruction getTransformInstruction() {
-		return transformInstruction;
-	}
-
 	@Override
 	public Object getUserData(String key) {
 
@@ -1140,6 +1193,7 @@ public class NodeImpl implements Node {
 	}
 
 	public void setNodeName(String nodeName) {
+		
 		this.nodeName = nodeName;
 	}
 
@@ -1163,7 +1217,8 @@ public class NodeImpl implements Node {
 
 	@Override
 	public void setPrefix(String prefix) throws DOMException {
-		System.err.println("Not yet implemented");
+		this.prefix = prefix;
+		
 	}
 
 	@Override
@@ -1181,11 +1236,6 @@ public class NodeImpl implements Node {
 		}
 	}
 
-	public void setTransformInstruction(
-			TransformInstruction transformInstruction) {
-		this.transformInstruction = transformInstruction;
-	}
-
 	@Override
 	public Object setUserData(String key, Object data, UserDataHandler handler) {
 
@@ -1329,8 +1379,6 @@ public class NodeImpl implements Node {
 	}
 
 	protected boolean validateAttributes() {
-
-		// To be overwritten
 		return true;
 	}
 
@@ -1381,6 +1429,22 @@ public class NodeImpl implements Node {
 		return true;
 	}
 
+	@SuppressWarnings("unchecked")
+	public <T> T getAnnotation(Class<T> type) {
+		
+		if (annotationList == null) {
+			return null;
+		}
+		
+		List<Annotation> collected = annotationList.stream().filter(annotation -> type.isInstance(annotation)).collect(Collectors.toList());
+		
+		if (collected != null && collected.size() > 0) {
+			return (T) collected.get(0);
+		}
+		
+		return null;
+	}
+	
 	public void vanish() {
 
 		if (parent != null) {
@@ -1390,4 +1454,22 @@ public class NodeImpl implements Node {
 		}
 	}
 
+	@SuppressWarnings("unchecked")
+	public <T> List<T> getAnnotations(Class<T> type) {
+
+		ArrayList<T> foundList = new ArrayList<>();
+		
+		if (annotationList == null) {
+			return foundList;
+		}
+		
+		List<Annotation> collected = annotationList.stream().filter(annotation -> type.isInstance(annotation)).collect(Collectors.toList());
+		
+		if (collected != null && collected.size() > 0) {
+			return (List<T>) collected;
+		}
+		
+		return foundList;
+	}
+
 }
